'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var socketio = _interopDefault(require('socket.io-client'));
var Logger = _interopDefault(require('nightingale-logger'));

/* eslint-disable no-use-before-define, @typescript-eslint/no-use-before-define, max-lines */
var logger = new Logger('alp:websocket');
var socket;
var successfulConnection = null;
var connected = false;

var callbackFirstConnectionError = function callbackFirstConnectionError() {
  successfulConnection = false;
};

var websocket = {
  get connected() {
    return connected;
  },

  on: on,
  off: off,
  emit: emit,
  isConnected: isConnected,
  isDisconnected: isDisconnected,
  socket: socket
};

function start(app, namespaceName) {
  var config = app.config,
      context = app.context;

  if (socket) {
    throw new Error('WebSocket already started');
  }

  var webSocketConfig = config.get('webSocket') || config.get('websocket');

  if (!webSocketConfig) {
    throw new Error('Missing config webSocket');
  }

  if (!webSocketConfig.has('port')) {
    throw new Error('Missing config webSocket.port');
  }

  var secure = webSocketConfig.get('secure');
  var port = webSocketConfig.get('port');
  var createdSocket = socketio("http" + (secure ? 's' : '') + "://" + window.location.hostname + ":" + port + "/" + namespaceName, {
    reconnectionDelay: 500,
    reconnectionDelayMax: 2500,
    timeout: 4000,
    transports: ['websocket']
  });
  socket = createdSocket;
  createdSocket.on('connect_error', callbackFirstConnectionError);
  createdSocket.on('connect', function () {
    createdSocket.off('connect_error', callbackFirstConnectionError);
    logger.success('connected');
    successfulConnection = true;
    connected = true;
  });
  createdSocket.on('reconnect', function () {
    logger.success('reconnected');
    connected = true;
  });
  createdSocket.on('disconnect', function () {
    logger.warn('disconnected');
    connected = false;
  });
  createdSocket.on('hello', function (_ref) {
    var version = _ref.version;

    if (version !== window.__VERSION__) {
      // eslint-disable-next-line no-alert
      if (window.confirm(context.t('newversion'))) {
        return window.location.reload(true);
      } else {
        console.warn('Version mismatch', {
          serverVersion: version,
          clientVersion: window.__VERSION__
        });
      }
    }
  });
  createdSocket.on('redux:action', function (action) {
    logger.debug('dispatch action from websocket', action); // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
    // @ts-ignore

    app.store.dispatch(action);
  });
  return createdSocket;
}

function emit(event) {
  for (_len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  if (!socket) throw new Error('Cannot call emit() before start()');

  var existingSocket = socket,
      _len,
      args,
      _key;

  logger.debug('emit', {
    args: args
  });
  return new Promise(function (resolve, reject) {
    var resolved = setTimeout(function () {
      logger.warn('websocket emit timeout', {
        args: args
      });
      reject(new Error('websocket response timeout'));
    }, 10000);
    existingSocket.emit.apply(existingSocket, [event].concat(args, [function (error, result) {
      clearTimeout(resolved);

      if (error != null) {
        return reject(typeof error === 'string' ? new Error(error) : error);
      }

      resolve(result);
    }]));
  });
}

function on(event, handler) {
  if (!socket) throw new Error('Cannot call on() before start()');
  socket.on(event, handler);
  return handler;
}

function off(event, handler) {
  if (!socket) throw new Error('Cannot call off() before start()');
  socket.off(event, handler);
}

function isConnected() {
  // socket.connected is not updated after reconnect event
  return !!socket && connected;
}

function isDisconnected() {
  return !!successfulConnection && !isConnected();
}

function alpWebsocket(app, namespaceName) {
  if (namespaceName === void 0) {
    namespaceName = '';
  }

  var socket = start(app, namespaceName);
  app.websocket = websocket;
  websocket.socket = socket;
  return socket;
}

exports.default = alpWebsocket;
exports.websocket = websocket;
//# sourceMappingURL=index-browser.cjs.js.map
